/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
<pre>{@code
                  y
                  |
                  |
                  | v[4]
                1 +---------------------+ v[5]=(1,1,0)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
       v[7] +---------------------+ v[6]|
            |     |               |     |
            |     |               |     |
            |     | v[0]          |     | v[1]
            |     +---------------|-----+------> x
            |    /                |    /1
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
          1 +---------------------+
           /v[3]=(0,0,1)          v[2]=(1,0,1)
          /
         /
        z
}</pre>
   Render a wireframe cube.
*/
public class InteractiveCube_R2
{
   private final FrameBufferPanel fbp; // The event handlers need
   private final Scene scene;          // access to these fields.
   private double deltaX = 0.0;
   private double deltaY = 0.0;
   private double deltaZ = 0.0;
   private double deltaS = 1.0;
   private boolean displayTransformations = false;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveCube_R2()
   {
      // Create the Scene object that we shall render.
      scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("cube");

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create the geometry for the Model.
      // Vertices.
      model.addVertex(new Vertex(0.0, 0.0, 0.0), // four vertices around the bottom face
                      new Vertex(1.0, 0.0, 0.0),
                      new Vertex(1.0, 0.0, 1.0),
                      new Vertex(0.0, 0.0, 1.0),
                      new Vertex(0.0, 1.0, 0.0), // four vertices around the top face
                      new Vertex(1.0, 1.0, 0.0),
                      new Vertex(1.0, 1.0, 1.0),
                      new Vertex(0.0, 1.0, 1.0));
      // Lines segments.
      model.addLineSegment(new LineSegment(0, 1),  // bottom face
                           new LineSegment(1, 2),  // bottom face
                           new LineSegment(2, 3),  // bottom face
                           new LineSegment(3, 0),  // bottom face
                           new LineSegment(4, 5),  // top face
                           new LineSegment(5, 6),  // top face
                           new LineSegment(6, 7),  // top face
                           new LineSegment(7, 4),  // top face
                           new LineSegment(0, 4),  // back face
                           new LineSegment(1, 5),  // back face
                           new LineSegment(3, 7),  // front face
                           new LineSegment(2, 6)); // front face

      // Push the model away from where the camera is.
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x,
                                            v.y,
                                            v.z - 2));
      }


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create (inner) event handler objects for events from the JFrame.
      jf.addKeyListener(new KeyHandler());
      jf.addComponentListener(new ComponentHandler());
   }


   // Define an inner KeyListener class.
   class KeyHandler extends KeyAdapter
   {
      @Override public void keyTyped(KeyEvent e)
      {
         //System.out.println( e );

         final char c = e.getKeyChar();
         if ('h' == c)
         {
            print_help_message();
            return;
         }
         else if ('d' == c)
         {
            Pipeline.debug = ! Pipeline.debug;
          //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
         }
         else if ('c' == c)
         {
            Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
            System.out.print("Clipping is turned ");
            System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
         }
         else if ('p' == c)
         {
            scene.camera.perspective = ! scene.camera.perspective;
            final String p = scene.camera.perspective ? "perspective" : "orthographic";
            System.out.println("Using " + p + " projection");
         }
         else if ('m' == c) //display transformation information
         {
            displayTransformations = ! displayTransformations;
         }
         else if ('s' == c) // Scale the cube 10% smaller.
         {
            scaleModel(1.0/1.1); // does NOT work with perspective projection
         }
         else if ('S' == c) // Scale the cube 10% larger.
         {
            scaleModel(1.1); // does NOT work with perspective projection
         }
         else if ('x' == c) // Translate the cube.
         {
            moveModel(-0.1, 0, 0); // left
         }
         else if ('X' == c)
         {
            moveModel(+0.1, 0, 0); // right
         }
         else if ('y' == c)
         {
            moveModel(0, -0.1, 0); // down
         }
         else if ('Y' == c)
         {
            moveModel(0, +0.1, 0);  // up
         }
         else if ('z' == c)
         {
            moveModel(0, 0, -0.1);  // back
         }
         else if ('Z' == c)
         {
            moveModel(0, 0, +0.1); // forward
         }

         if (displayTransformations && ('m'==c
              ||'s'==c||'x'==c||'y'==c||'z'==c
              ||'S'==c||'X'==c||'Y'==c||'Z'==c))
         {
            System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                              "deltaZ = %.2f, deltaS = %.2f\n",
                               deltaX,        deltaY,
                               deltaZ,        deltaS);
         }

         // Render again.
         final FrameBuffer fb = fbp.getFrameBuffer();
         fb.clearFB();
         Pipeline.render(scene, fb);
         fbp.update();
      }

      private void scaleModel(double deltaS)
      {
         InteractiveCube_R2.this.deltaS *= deltaS;
         final Model m = scene.getModel(0);
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x * deltaS,
                                           v.y * deltaS,
                                           v.z * deltaS));
         }
      }


      private void moveModel(double deltaX, double deltaY, double deltaZ)
      {
         InteractiveCube_R2.this.deltaX += deltaX;
         InteractiveCube_R2.this.deltaY += deltaY;
         InteractiveCube_R2.this.deltaZ += deltaZ;
         final Model m = scene.getModel(0);
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x + deltaX,
                                           v.y + deltaY,
                                           v.z + deltaZ));
         }
      }
   }


   // Define an inner ComponentListener class.
   class ComponentHandler extends ComponentAdapter
   {
      @Override public void componentResized(ComponentEvent e)
      {
         //System.out.println( e );

         System.out.printf("JFrame [w = %d, h = %d]: " +
                           "FrameBufferPanel [w = %d, h = %d].\n",
                           fbp.getTopLevelAncestor().getWidth(),
                           fbp.getTopLevelAncestor().getHeight(),
                           fbp.getWidth(), fbp.getHeight());

         // Get the new size of the FrameBufferPanel.
         final int w = fbp.getWidth();
         final int h = fbp.getHeight();

         // Create a new FrameBuffer that fits the FrameBufferPanel.
         final FrameBuffer fb = new FrameBuffer(w, h);
         fbp.setFrameBuffer(fb);
         Pipeline.render(scene, fb);
         fbp.update();
      }
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveCube_R2()
      );
   }


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the cube along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the cube.");
      System.out.println("Use the 'm' key to toggle the display of transformation information.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
