/*

*/

import renderer.scene.*;
import renderer.framebuffer.FrameBufferPanel;

import javax.swing.JFrame;
import java.awt.BorderLayout;

/**
<pre>{@code
                  y
                  |
                  |
                  | v[4]
                1 +---------------------+ v[5]=(1,1,0)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
       v[7] +---------------------+ v[6]|
            |     |               |     |
            |     |               |     |
            |     | v[0]          |     | v[1]
            |     +---------------|-----+------> x
            |    /                |    /1
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
          1 +---------------------+
           /v[3]=(0,0,1)          v[2]=(1,0,1)
          /
         /
        z
}</pre>
   Render a wireframe cube.
*/
public class InteractiveCube_R2
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public InteractiveCube_R2()
   {
      // Create an object that can hold all the client state information.
      final ClientState state = new ClientState();

      // Create the Scene object that we shall render.
      state.scene = new Scene();

      // Create a Model object to hold the geometry.
      final Model model = new Model("cube");
      state.modelArray.add( model );

      // Add the Model to the Scene.
      state.scene.addModel(state.modelArray.get(0));

      // Create the geometry for the Model.
      // Vertices.
      model.addVertex(new Vertex(0.0, 0.0, 0.0), // four vertices around the bottom face
                      new Vertex(1.0, 0.0, 0.0),
                      new Vertex(1.0, 0.0, 1.0),
                      new Vertex(0.0, 0.0, 1.0),
                      new Vertex(0.0, 1.0, 0.0), // four vertices around the top face
                      new Vertex(1.0, 1.0, 0.0),
                      new Vertex(1.0, 1.0, 1.0),
                      new Vertex(0.0, 1.0, 1.0));
      // Lines segments.
      model.addLineSegment(new LineSegment(0, 1),  // bottom face
                           new LineSegment(1, 2),  // bottom face
                           new LineSegment(2, 3),  // bottom face
                           new LineSegment(3, 0),  // bottom face
                           new LineSegment(4, 5),  // top face
                           new LineSegment(5, 6),  // top face
                           new LineSegment(6, 7),  // top face
                           new LineSegment(7, 4),  // top face
                           new LineSegment(0, 4),  // back face
                           new LineSegment(1, 5),  // back face
                           new LineSegment(3, 7),  // front face
                           new LineSegment(2, 6)); // front face

      // Push the model away from where the camera is.
      for (int i = 0; i < model.vertexList.size(); ++i)
      {
         final Vertex v = model.vertexList.get(i);
         model.vertexList.set(i, new Vertex(v.x,
                                            v.y,
                                            v.z - 2));
      }


      // Define initial dimensions for a FrameBuffer.
      final int width  = 1024;
      final int height = 1024;

      // Create a FrameBufferPanel that holds a FrameBuffer.
      state.fbp = new FrameBufferPanel(width, height);

      // Create a JFrame that will hold the FrameBufferPanel.
      final JFrame jf = new JFrame("Renderer 2");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      jf.setLocationByPlatform(true);
      // Place the FrameBufferPanel in the JFrame.
      jf.getContentPane().add(state.fbp, BorderLayout.CENTER);
      jf.pack();
      jf.setVisible(true);

      // Create event handler objects for events from the JFrame.
      jf.addKeyListener(new KeyHandler(state));
      jf.addComponentListener(new ComponentHandler(state));
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      ClientState.print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new InteractiveCube_R2()
      );
   }
}
