/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.util.List;
import java.util.ArrayList;

/**

*/
public abstract class InteractiveAbstractClient_R2 implements KeyListener, ComponentListener
{
   protected FrameBufferPanel fbp;
   protected Scene scene;
   protected final List<Model> modelArray = new ArrayList<>();
   protected int currentModel = 0;
   protected double[] deltaX = {0.0};
   protected double[] deltaY = {0.0};
   protected double[] deltaZ = {0.0};
   protected double[] deltaS = {1.0};
   protected boolean displayTransformations = false;


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      final char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         modelArray.get(currentModel).debug = ! modelArray.get(currentModel).debug;
       //Rasterize_Clip.debug = ! Rasterize_Clip.debug;
      }
      else if ('c' == c)
      {
         Rasterize_Clip.doClipping = ! Rasterize_Clip.doClipping;
         System.out.print("Clipping is turned ");
         System.out.println(Rasterize_Clip.doClipping ? "On" : "Off");
      }
      else if ('/' == c)
      {
         currentModel = (currentModel + 1) % modelArray.size();
         scene.setModel( 0, modelArray.get(currentModel) );
      }
      else if ('?' == c)
      {
         currentModel = (currentModel - 1);
         if (currentModel < 0) currentModel = modelArray.size() - 1;
         scene.setModel( 0, modelArray.get(currentModel) );
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
         final String p = scene.camera.perspective ? "perspective" : "orthographic";
         System.out.println("Using " + p + " projection");
      }
      else if ('m' == c) //display transformation information
      {
         displayTransformations = ! displayTransformations;
      }
      else if ('s' == c) // Scale the current model 10% smaller.
      {
         // This does NOT work with perspective projection.
         scaleModel(currentModel, 1.0/1.1);
      }
      else if ('S' == c) // Scale the current model 10% larger.
      {
         // This does NOT work with perspective projection.
         scaleModel(currentModel, 1.1);
      }
      else if ('x' == c) // Translate the current model.
      {
         moveModel(currentModel, -0.1, 0, 0); // left
      }
      else if ('X' == c)
      {
         moveModel(currentModel, +0.1, 0, 0); // right
      }
      else if ('y' == c)
      {
         moveModel(currentModel, 0, -0.1, 0); // down
      }
      else if ('Y' == c)
      {
         moveModel(currentModel, 0, +0.1, 0); // up
      }
      else if ('z' == c)
      {
         moveModel(currentModel, 0, 0, -0.1); // back
      }
      else if ('Z' == c)
      {
         moveModel(currentModel, 0, 0, +0.1); // forward
      }

      if (displayTransformations &&
            ('m'==c||'/'==c||'?'==c
           ||'s'==c||'x'==c||'y'==c||'z'==c
           ||'S'==c||'X'==c||'Y'==c||'Z'==c))
      {
         System.out.printf("deltaX = %.2f, deltaY = %.2f, " +
                           "deltaZ = %.2f, deltaS = %.2f\n",
                           deltaX[currentModel], deltaY[currentModel],
                           deltaZ[currentModel], deltaS[currentModel]);
      }

      // Render again.
      final FrameBuffer fb = fbp.getFrameBuffer();
      fb.clearFB();
      Pipeline.render(scene, fb);
      fbp.update();
   }


   protected void scaleModel(int currentModel, double deltaS)
   {
      this.deltaS[currentModel] *= deltaS;
      final Model m = scene.getModel(0);
      for (int i = 0; i < m.vertexList.size(); ++i)
      {
         final Vertex v = m.vertexList.get(i);
         m.vertexList.set(i, new Vertex(v.x * deltaS,
                                        v.y * deltaS,
                                        v.z * deltaS));
      }
   }


   protected void moveModel(int currentModel, double deltaX, double deltaY, double deltaZ)
   {
      this.deltaX[currentModel] += deltaX;
      this.deltaY[currentModel] += deltaY;
      this.deltaZ[currentModel] += deltaZ;
      final Model m = scene.getModel(0);
      for (int i = 0; i < m.vertexList.size(); ++i)
      {
         final Vertex v = m.vertexList.get(i);
         m.vertexList.set(i, new Vertex(v.x + deltaX,
                                        v.y + deltaY,
                                        v.z + deltaZ));
      }
   }


   // Implement the ComponentListener interface.
   @Override public void componentMoved(ComponentEvent e){}
   @Override public void componentHidden(ComponentEvent e){}
   @Override public void componentShown(ComponentEvent e){}
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      System.out.printf("JFrame [w = %d, h = %d]: " +
                        "FrameBufferPanel [w = %d, h = %d].\n",
                        fbp.getTopLevelAncestor().getWidth(),
                        fbp.getTopLevelAncestor().getHeight(),
                        fbp.getWidth(), fbp.getHeight());

      // Get the new size of the FrameBufferPanel.
      final int w = fbp.getWidth();
      final int h = fbp.getHeight();

      // Create a new FrameBuffer that fits the FrameBufferPanel.
      final FrameBuffer fb = new FrameBuffer(w, h);
      fbp.setFrameBuffer(fb);
      Pipeline.render(scene, fb);
      fbp.update();
   }


   protected static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off for the current model.");
      System.out.println("Use the '/' key to cycle through the models.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the current model along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the current model.");
      System.out.println("Use the 'm' key to toggle the display of transformation information.");
      System.out.println("Use the 'c' key to toggle line clipping on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
