/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a cuboid aligned with
   the x, y, and z axes and with one corner at the
   origin.
<p>
   Here is a picture showing how the cuboid's eight
   corners are labelled.
<pre>{@code
                  y
                  |
                  |
                  | (0,0,ys)
                  +---------------------+ (xs,0,ys)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |                /    |
  (0,ys,zs) +---------------------+     |
            |     |               |     |
            |     |               |     |
            |     | (0,0,0)       |     | (xs,0,0)
            |     +---------------|-----+------> x
            |    /                |    /
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
            +---------------------+
           / (0,0,zs)              (xs,0,zs)
          /
         /
        z
}</pre>
   See <a href="http://en.wikipedia.org/wiki/Cuboid" target="_top">
                http://en.wikipedia.org/wiki/Cuboid</a>

   @see PointCloud_Cube
*/
public class PointCloud_Box extends Model
{
   /**
      Create a {@code PointCloud_Box} with all three sides of length 1.
   */
   public PointCloud_Box( )
   {
      this(1, 1, 1, 100);
   }


   /**
      Create a {@code PointCloud_Box} with the given side lengths.

      @param xs  the size of the {@code Box} along the x-axis
      @param ys  the size of the {@code Box} along the y-axis
      @param zs  the size of the {@code Box} along the z-axis
      @param n   number of points along each edge of the box
   */
   public PointCloud_Box(final double xs, final double ys, final double zs, final int n)
   {
      super("Point Cloud Box");

      // Create 8 vertices.
      addVertex(new Vertex(0,    0,    0), // 4 vertices around the bottom face
                new Vertex(0+xs, 0,    0),
                new Vertex(0+xs, 0,    0+zs),
                new Vertex(0,    0,    0+zs),
                new Vertex(0,    0+ys, 0), // 4 vertices around the top face
                new Vertex(0+xs, 0+ys, 0),
                new Vertex(0+xs, 0+ys, 0+zs),
                new Vertex(0,    0+ys, 0+zs));

      // Create the geometry for the Model.
      final double deltaX = xs/n;
      final double deltaY = ys/n;
      final double deltaZ = zs/n;

      // Vertices.
      for (int i = 0; i <= n; ++i)
      {
         // back face
         addVertex(new Vertex(0.0,       i*deltaY,  0.0),
                   new Vertex(xs,        i*deltaY,  0.0),
                   new Vertex(i*deltaX,  0.0,       0.0),
                   new Vertex(i*deltaX,  ys,        0.0));
         // front face
         addVertex(new Vertex(0.0,       i*deltaY,  zs),
                   new Vertex(xs,        i*deltaY,  zs),
                   new Vertex(i*deltaX,  0.0,       zs),
                   new Vertex(i*deltaX,  ys,        zs));
         // four edges from front to back
         addVertex(new Vertex(0.0, 0.0,  i*deltaZ),
                   new Vertex(xs,  0.0,  i*deltaZ),
                   new Vertex(xs,   ys,  i*deltaZ),
                   new Vertex(0.0,  ys,  i*deltaZ));
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Box
