/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a frustum of a right circular cone
   with its base in the xz-plane.

   @see PointCloudCone
   @see PointCloudConeSector
*/
public class PointCloud_ConeFrustum extends Model
{
   /**
      Create a frustum of a right circular cone with its base in the
      xz-plane, a base radius of 1, top radius of 1/2, and height 1/2.
   */
   public PointCloud_ConeFrustum( )
   {
      this(1.0, 0.5, 0.5, 100, 100);
   }


   /**
      Create a frustum of a right circular cone with its base in the
      xz-plane, a base radius of {@code r}, top of the frustum at
      height {@code h}, and with the cone's apex on the y-axis at
      height {@code a}.

      @param n  number of circles of latitude
      @param k  number of lines of longitude
      @param r  radius of the base in the xz-plane
      @param h  height of the frustum
      @param a  height of the apex of the cone
   */
   public PointCloud_ConeFrustum(final int n, final int k,
                                 final double r, final double h,
                                 final double a)
   {
      this(r, (1 - h/a)*r, h, n, k);
   }


   /**
      Create a frustum of a right circular cone with its base in the
      xz-plane, a base radius of {@code r1}, top radius of {@code r2},
      and height {@code h}.
   <p>
      This model works with either {@code r1 > r2} or {@code r1 < r2}.
      In other words, the frustum can have its "apex" either above or
      below the xz-plane.

      @param r1  radius of the base of the frustum
      @param h   height of the frustum
      @param r2  radius of the top of the frustum
      @param n   number of circles of latitude
      @param k   number of lines of longitude
   */
   public PointCloud_ConeFrustum(final double r1,
                                 final double h,
                                 final double r2,
                                 final int n, final int k)
   {
      super("Point Cloud Cone Frustum");

      // Create the frustum's geometry.
      final double deltaTheta = (2 * Math.PI) / k;

      // Vertices.
      for (int j = 0; j < k; ++j) // choose an angle of longitude
      {
         double c = Math.cos(j * deltaTheta);
         double s = Math.sin(j * deltaTheta);
         for (int i = 0; i < n; ++i) // choose a circle of latitude
         {
            double slantRadius = (i/(double)(n-1)) * r1 + ((n-1-i)/(double)(n-1)) * r2;
            addVertex( new Vertex(slantRadius * c,
                                  h - (i*h)/(n-1),
                                  slantRadius * s) );
         }
      }
      addVertex( new Vertex(0, h, 0) );  // top center
      addVertex( new Vertex(0, 0, 0) );  // bottom center

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_ConeFrustum
