/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a cube with its center
   at the origin, having edge length 2, and with its
   vertices at {@code (1, 1, 1)}.
<p>
   Here is a picture showing the cube's eight corners.
<pre>{@code

        (-1,1,-1) +---------------------+ (1,1,-1)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |        (1,1,1) /    |
   (-1,1,1) +---------------------+     |
            |     |               |     |
            |     |               |     |
            |     | (-1,-1,-1)    |     |
            |     +---------------|-----+ (1,-1,-1)
            |    /                |    /
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
            +---------------------+
           (-1,-1,1)               (1,-1,1)
}</pre>
   See <a href="https://en.wikipedia.org/wiki/Cube" target="_top">
                https://en.wikipedia.org/wiki/Cube</a>

   @see PointCloud_Tetrahedron
   @see PointCloud_Octahedron
   @see PointCloud_Icosahedron
   @see PointCloud_Dodecahedron
*/
public class PointCloud_Cube extends Model
{
   /**
      Create a cube with 100 points along each edge.
   */
   public PointCloud_Cube( )
   {
      this(100);
   }


   /**
      Create a cube with {@code n} points along each edge.

      @param n  number of points along each edge of the cube
   */
   public PointCloud_Cube(final int n)
   {
      super("Point Cloud Cube (n="+n+")");

      // Create the geometry for the Model.
      final double delta = 2.0/n;

      // Vertices.
      for (int i = 0; i <= n; ++i)
      {
         // back face
         addVertex(new Vertex( 1.0,             1.0 - i*delta,  -1.0),
                   new Vertex( 1.0 - i*delta,  -1.0,            -1.0),
                   new Vertex(-1.0,            -1.0 + i*delta,  -1.0),
                   new Vertex(-1.0 + i*delta,   1.0,            -1.0));
         // front face
         addVertex(new Vertex( 1.0,             1.0 - i*delta,   1.0),
                   new Vertex( 1.0 - i*delta,  -1.0,             1.0),
                   new Vertex(-1.0,            -1.0 + i*delta,   1.0),
                   new Vertex(-1.0 + i*delta,   1.0,             1.0));
         // four edges from front to back
         addVertex(new Vertex( 1.0,  1.0,  1.0 - i*delta),
                   new Vertex( 1.0, -1.0,  1.0 - i*delta),
                   new Vertex(-1.0, -1.0,  1.0 - i*delta),
                   new Vertex(-1.0,  1.0,  1.0 - i*delta));
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Cube
