/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a "solid" point cloud model of a cube with its center
   at the origin, having edge length 2, and with its
   vertices at {@code (1, 1, 1)}.
<p>
   Here is a picture showing the cube's eight corners.
<pre>{@code

        (-1,1,-1) +---------------------+ (1,1,-1)
                 /|                    /|
                / |                   / |
               /  |                  /  |
              /   |                 /   |
             /    |         (1,1,1)/    |
   (-1,1,1) +---------------------+     |
            |     |               |     |
            |     |               |     |
            |     | (-1,-1,-1)    |     |
            |     +---------------|-----+ (1,-1,-1)
            |    /                |    /
            |   /                 |   /
            |  /                  |  /
            | /                   | /
            |/                    |/
            +---------------------+
           (-1,-1,1)             (1,-1,1)
}</pre>
*/
public class PointCloud_CubeSolid extends Model
{
   /**
      Create a "solid" cube with 100 points along each edge.
   */
   public PointCloud_CubeSolid( )
   {
      this(100);
   }


   /**
      Create a "solid" cube with {@code n} points along each edge.

      @param n  number of points along each edge of the cube
   */
   public PointCloud_CubeSolid(int n)
   {
      super("Point Cloud Solid Cube (n="+n+")");

      // Create the geometry for the Model.
      final double delta = 2.0/n;

      // Vertices.
      for (int i = 0; i < n; ++i)
      {
         // move from the front face towards the back
         for (int j = 0; j < n; ++j)
         {
            addVertex(new Vertex( 1.0,             1.0 - i*delta,  1 - j*delta),
                      new Vertex( 1.0 - i*delta,  -1.0,            1 - j*delta),
                      new Vertex(-1.0,            -1.0 + i*delta,  1 - j*delta),
                      new Vertex(-1.0 + i*delta,   1.0,            1 - j*delta));
         }
      }
      // back and front faces
      for (int i = 0; i < n; ++i)
      {
         for (int j = 0; j < n; ++j)
         {
            addVertex(new Vertex(-1 + i*delta, -1 + j*delta, -1.0)); // back face
          //addVertex(new Vertex(-1 + i*delta, -1 + j*delta,  1.0)); // front face
         }
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_CubeSolid
