/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a partial right circular cylinder
   with its axis along the y-axis.
<p>
   By a partial cylinder we mean a cylinder over a circular sector
   of the cylinder's base.

   @see PointCloudCylinder
*/
public class PointCloud_CylinderSector extends Model
{
   /**
      Create half of a cylinder with radius 1
      and its axis along the y-axis from
      {@code y = -1} to {@code y = 1}.
   */
   public PointCloud_CylinderSector( )
   {
      this(1, 1, Math.PI/2, 3*Math.PI/2, 15, 8);
   }


   /**
      Create a part of the cylinder with radius {@code r} and its
      axis along the y-axis from {@code y = -h} to {@code y = h}.
   <p>
      The partial cylinder is a cylinder over the circular sector
      from angle {@code theta1} to angle {@code theta2}.
   <p>
      The last two parameters determine the number of lines of longitude
      and the number of (partial) circles of latitude in the model.

      @param r       radius of the cylinder
      @param h       height of the cylinder
      @param theta1  beginning longitude angle of the sector
      @param theta2  ending longitude angle of the sector
      @param n       number of circles of latitude around the cylinder
      @param k       number of lines of longitude
   */
   public PointCloud_CylinderSector(final double r, final double h,
                                    final double theta1, final double theta2,
                                    final int n, final int k)
   {
      super("Point Cloud Cylinder Sector");

      // Create the cylinder's geometry.
      final double deltaH = (2 * h) / (n - 1);
      final double deltaTheta = (theta2 - theta1)/ (k - 1);

      // Vertices.
      for (int j = 0; j < k; ++j) // choose an angle of longitude
      {
         final double c = Math.cos(theta1 + j*deltaTheta);
         final double s = Math.sin(theta1 + j*deltaTheta);
         for (int i = 0; i < n; ++i) // choose a circle of latitude
         {
            addVertex(new Vertex( r * c,
                                 -h + i * deltaH,
                                  r * s ) );
         }
      }
      addVertex(new Vertex(0,  h, 0));  // top center
      addVertex(new Vertex(0, -h, 0));  // bottom center

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_CylinderSector
