/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a sector of a disk
   in the xy-plane centered at the origin.
<p>
   See <a href="https://en.wikipedia.org/wiki/Circular_sector" target="_top">
                https://en.wikipedia.org/wiki/Circular_sector</a>

   @see PointCloud_Disk
*/
public class PointCloud_DiskSector extends Model
{
   /**
      Create half a disk in the xy-plane with radius 1,
      and with 50 points around the circumference.
   */
   public PointCloud_DiskSector( )
   {
      this(1, 0, Math.PI, 100, 100);
   }


   /**
      Create half a disk in the xy-plane with radius {@code r}
      and with {@code n} points around the circumference.

      @param r  radius of the disk
   */
   public PointCloud_DiskSector(final double r)
   {
      this(r, 0, Math.PI, 100, 100);
   }


   /**
      Create a sector of a disk in the xy-plane with radius
      {@code r}, with {@code n} spokes coming out of the center,
      and with {@code k} concentric circles around the disk.

      @param r       radius of the disk
      @param theta1  beginning angle of the sector
      @param theta2  ending angle of the sector
      @param n       number of points in the disk's (partial) circumference
      @param k       number of points in the disk's spokes
   */
   public PointCloud_DiskSector(final double r,
                                final double theta1,
                                final double theta2,
                                final int n, final int k)
   {
      super("Point Cloud Disk Sector");

      // Create the disk's geometry.
      final double deltaTheta = (theta2 - theta1) / n;
      final double deltaR = r / k;

      // Vertices.
      for (int j = 0; j <= n; ++j) // choose a spoke (an angle)
      {
         final double c = Math.cos(theta1 + j * deltaTheta);
         final double s = Math.sin(theta1 + j * deltaTheta);
         for (int i = 1; i <= k; ++i) // move along the spoke
         {
            final double ri = i * deltaR;
            addVertex(new Vertex( ri * c,
                                  ri * s,
                                  0 ) );
         }
      }
      addVertex(new Vertex(0,0,0)); // center

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//DiskSector
