/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a regular dodecahedron
   with its center at the origin, having edge length
   <pre>{@code
     2*(sqrt(5)-1)/(1+sqrt(5)) = (1/2)*(sqrt(5)-1)^2 = 0.7639,
   }</pre>
   and with its vertices on a sphere of radius
   <pre>{@code
     2*sqrt(3)/(1+sqrt(5)) = 1.0705.
   }</pre>
<p>
   See <a href="https://en.wikipedia.org/wiki/Regular_dodecahedron" target="_top">
                https://en.wikipedia.org/wiki/Regular_dodecahedron</a>

   @see PointCloudTetrahedron
   @see PointCloudCube
   @see PointCloudOctahedron
   @see PointCloudIcosahedron
*/
public class PointCloud_Dodecahedron extends Model
{
   /**
      Create a regular dodecahedron with its center at
      the origin, having edge length
      <pre>{@code
        2*(sqrt(5)-1)/(1+sqrt(5)) = (1/2)*(sqrt(5)-1)^2 = 0.7639,
      }</pre>
      and with its vertices on a sphere of radius
      <pre>{@code
        2*sqrt(3)/(1+sqrt(5)) = 1.0705.
     }</pre>
   */
   public PointCloud_Dodecahedron()
   {
      this(100);
   }


   /**
      Create a regular dodecahedron with {@code n}
      points along each edge.

      @param n  number of points along each edge
   */
   public PointCloud_Dodecahedron(final int n)
   {
      super("Point Cloud Dodecahedron");

      // Create the dodecahedron's geometry.
      // It has 20 vertices and 30 edges.
      final double t = (1 + Math.sqrt(5))/2;   // golden ratio
      final double r = 1/t;
      final double r2 = r * r;
      //https://en.wikipedia.org/wiki/Regular_dodecahedron#Cartesian_coordinates
      Vertex v00, v01, v02, v03, v04, v05, v06, v07, v08, v09,
             v10, v11, v12, v13, v14, v15, v16, v17, v18, v19;

      // (r, r, r)
      v00 = new Vertex(-r, -r, -r);
      v01 = new Vertex(-r, -r,  r);
      v02 = new Vertex(-r,  r, -r);
      v03 = new Vertex(-r,  r,  r);
      v04 = new Vertex( r, -r, -r);
      v05 = new Vertex( r, -r,  r);
      v06 = new Vertex( r,  r, -r);
      v07 = new Vertex( r,  r,  r);

      // (0, r2, 1)
      v08 = new Vertex( 0, -r2, -1);
      v09 = new Vertex( 0, -r2,  1);
      v10 = new Vertex( 0,  r2, -1);
      v11 = new Vertex( 0,  r2,  1);

      // (r2, 1, 0)
      v12 = new Vertex(-r2, -1,  0);
      v13 = new Vertex(-r2,  1,  0);
      v14 = new Vertex( r2, -1,  0);
      v15 = new Vertex( r2,  1,  0);

      // (1, 0, r2)
      v16 = new Vertex(-1,  0, -r2);
      v17 = new Vertex( 1,  0, -r2);
      v18 = new Vertex(-1,  0,  r2);
      v19 = new Vertex( 1,  0,  r2);
/*
      // These vertices create a dodecahedron with vertices
      // on a sphere of radius sqrt(3), and with edge length
      //    2/t = 4/(1 + sqrt(5)) = sqrt(5) - 1 = 1.2361.
      //https://en.wikipedia.org/wiki/Regular_dodecahedron#Cartesian_coordinates
      // (1, 1, 1)
      v00 = new Vertex(-1, -1, -1);
      v01 = new Vertex(-1, -1,  1);
      v02 = new Vertex(-1,  1, -1);
      v03 = new Vertex(-1,  1,  1);
      v04 = new Vertex( 1, -1, -1);
      v05 = new Vertex( 1, -1,  1);
      v06 = new Vertex( 1,  1, -1);
      v07 = new Vertex( 1,  1,  1);

      // (0, r, t)
      v08 = new Vertex( 0, -r, -t);
      v09 = new Vertex( 0, -r,  t);
      v10 = new Vertex( 0,  r, -t);
      v11 = new Vertex( 0,  r,  t);

      // (r, t, 0)
      v12 = new Vertex(-r, -t,  0);
      v13 = new Vertex(-r,  t,  0);
      v14 = new Vertex( r, -t,  0);
      v15 = new Vertex( r,  t,  0);

      // (t, 0, r)
      v16 = new Vertex(-t,  0, -r);
      v17 = new Vertex( t,  0, -r);
      v18 = new Vertex(-t,  0,  r);
      v19 = new Vertex( t,  0,  r);
*/

      addVertex(v00, v01, v02, v03, v04, v05, v06, v07, v08, v09,
                v10, v11, v12, v13, v14, v15, v16, v17, v18, v19);

      // Create 30 line segments (that make up 12 faces).
//https://github.com/mrdoob/three.js/blob/master/src/geometries/DodecahedronGeometry.js
      addEdge(v03, v11, n);
      addEdge(v11, v07, n);
      addEdge(v07, v15, n);
      addEdge(v15, v13, n);
      addEdge(v13, v03, n);

      addEdge(v07, v19, n);
      addEdge(v19, v17, n);
      addEdge(v17, v06, n);
      addEdge(v06, v15, n);

      addEdge(v17, v04, n);
      addEdge(v04, v08, n);
      addEdge(v08, v10, n);
      addEdge(v10, v06, n);

      addEdge(v08, v00, n);
      addEdge(v00, v16, n);
      addEdge(v16, v02, n);
      addEdge(v02, v10, n);

      addEdge(v00, v12, n);
      addEdge(v12, v01, n);
      addEdge(v01, v18, n);
      addEdge(v18, v16, n);

      addEdge(v02, v13, n);

      addEdge(v18, v03, n);

      addEdge(v01, v09, n);
      addEdge(v09, v11, n);

      addEdge(v04, v14, n);
      addEdge(v14, v12, n);

      addEdge(v09, v05, n);
      addEdge(v05, v19, n);

      addEdge(v05, v14, n);

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between {@link Vertex} {@code v0} and {@link Vertex}
      {@code v1}.

      @param v0  starting {link Vertex} of a line segment
      @param v1  ending {link Vertex} of a line segment
      @param n   number of points for this edge
   */
   private void addEdge(final Vertex v0, final Vertex v1, final int n)
   {
      final double deltaT = 1.0 / (n - 1);

      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         t += deltaT;
      }
   }
}//PointCloud_Dodecahedron
