/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a regular icosahedron
   with its center at the origin, having edge length
   <pre>{@code
     4/(1+sqrt(5)) = 1.2361,
   }</pre>
   and with its vertices on a sphere of radius
   <pre>{@code
     4/(1+sqrt(5)) * sin(2Pi/5) = 1.1756.
   }</pre>
<p>
   See <a href="https://en.wikipedia.org/wiki/Regular_icosahedron" target="_top">
                https://en.wikipedia.org/wiki/Regular_icosahedron</a>

   @see PointCloudTetrahedron
   @see PointCloudCube
   @see PointCloudOctahedron
   @see PointCloudDodecahedron
*/
public class PointCloud_Icosahedron extends Model
{
   /**
      Create a regular icosahedron with its center at
      the origin, having edge length
      <pre>{@code
        4/(1+sqrt(5)) = 1.2361,
      }</pre>
      and with its vertices on a sphere of radius
      <pre>{@code
        4/(1+sqrt(5)) * sin(2Pi/5) = 1.1756.
      }</pre>
   */
   public PointCloud_Icosahedron()
   {
      this(100);
   }


   /**
      Create a regular icosahedron with {@code n}
      points along each edge.

      @param n  number of points along each edge
   */
   public PointCloud_Icosahedron(final int n)
   {
      super("Point Cloud Icosahedron");

      // Create the icosahedron's geometry.
      // It has 12 vertices and 30 edges.
      final double t = (1 + Math.sqrt(5))/2;  // golden ratio
      final double r = 1/t;
      //https://en.wikipedia.org/wiki/Regular_icosahedron#Cartesian_coordinates
      Vertex v00, v01, v02, v03, v04, v05, v06, v07, v08, v09, v10, v11;

      // All cyclic permutations of (0, r, 1).
      v00 = new Vertex(-r,  1,  0);
      v01 = new Vertex( r,  1,  0);
      v02 = new Vertex(-r, -1,  0);
      v03 = new Vertex( r, -1,  0);
      v04 = new Vertex( 0, -r,  1);
      v05 = new Vertex( 0,  r,  1);
      v06 = new Vertex( 0, -r, -1);
      v07 = new Vertex( 0,  r, -1);
      v08 = new Vertex( 1,  0, -r);
      v09 = new Vertex( 1,  0,  r);
      v10 = new Vertex(-1,  0, -r);
      v11 = new Vertex(-1,  0,  r);
/*
      // These vertices create a icosahedron with edge length 2,
      // and vertices on a sphere of radius
      //    sqrt(10+2sqrt(5))/2 = 2sin(2Pi/5) = 1.90211.
      //https://en.wikipedia.org/wiki/Regular_icosahedron#Cartesian_coordinates
      // and also
      //https://github.com/mrdoob/three.js/blob/master/src/geometries/IcosahedronGeometry.js
      // All cyclic permutations of (0, 1, t),
      v00 = new Vertex(-1,  t,  0);
      v01 = new Vertex( 1,  t,  0);
      v02 = new Vertex(-1, -t,  0);
      v03 = new Vertex( 1, -t,  0);
      v04 = new Vertex( 0, -1,  t);
      v05 = new Vertex( 0,  1,  t);
      v06 = new Vertex( 0, -1, -t);
      v07 = new Vertex( 0,  1, -t);
      v08 = new Vertex( t,  0, -1);
      v09 = new Vertex( t,  0,  1);
      v10 = new Vertex(-t,  0, -1);
      v11 = new Vertex(-t,  0,  1);
*/
      addVertex(v00, v01, v02, v03, v04, v05, v06, v07, v08, v09, v10, v11);

      // Create 30 line segments.
      // To figure out the edges, look at the orthogonal projection in the z-direction.
      // https://en.wikipedia.org/wiki/Regular_icosahedron#Orthogonal_projections

      // The edge from v00 to v01 is the top horizontal edge.
      // The edge from v02 to v03 is the bottom horizontal edge.
      // The edge from v04 to v05 is the front vertical edge.
      // The edge from v06 to v07 is the back vertical edge.
      // The edge from v08 to v09 is the right horizontal edge.
      // The edge from v10 to v11 is the left horizontal edge.

      // Working, more or less, from the top down.
      addEdge(v00, v01, n);
      addEdge(v00, v05, n);
      addEdge(v00, v07, n);
      addEdge(v00, v11, n);
      addEdge(v00, v10, n);
      addEdge(v01, v05, n);
      addEdge(v01, v07, n);
      addEdge(v01, v09, n);
      addEdge(v01, v08, n);
      addEdge(v05, v11, n);
      addEdge(v05, v09, n);
      addEdge(v05, v04, n);
      addEdge(v07, v10, n);
      addEdge(v07, v08, n);
      addEdge(v07, v06, n);
      addEdge(v11, v10, n);
      addEdge(v11, v04, n);
      addEdge(v11, v02, n);
      addEdge(v09, v08, n);
      addEdge(v09, v04, n);
      addEdge(v09, v03, n);
      addEdge(v10, v06, n);
      addEdge(v10, v02, n);
      addEdge(v08, v06, n);
      addEdge(v08, v03, n);
      addEdge(v04, v02, n);
      addEdge(v04, v03, n);
      addEdge(v06, v02, n);
      addEdge(v06, v03, n);
      addEdge(v02, v03, n);

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between {@link Vertex} {@code v0} and {@link Vertex}
      {@code v1}.

      @param v0  starting {link Vertex} of a line segment
      @param v1  ending {link Vertex} of a line segment
      @param n   number of points for this edge
   */
   private void addEdge(final Vertex v0, final Vertex v1, final int n)
   {
      final double deltaT = 1.0 / (n - 1);

      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         t += deltaT;
      }
   }
}//PointCloud_Icosahedron
