/*

*/

package models_pc;
import  renderer.scene.*;

import java.util.Scanner;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.regex.*;

/**
<p>
   A simple demonstration of loading and drawing a basic OBJ file
   as a solid point cloud.
<p>
   A basic OBJ file is a text file that contains three kinds of lines:
   lines that begin with the character {@code 'v'}, lines that begin
   with the character {@code 'f'}, and lines that begin with the
   character {@code '#'}.
<p>
   A line in an OBJ file that begins with {@code '#'} is a comment line
   and can be ignored.
<p>
   A line in an OBJ file that begins with {@code 'v'} is a line that
   describes a vertex in 3-dimensional space. The {@code 'v'} will always
   be followed on the line by three doubles, the {@code x}, {@code y},
   and {@code z} coordinates of the vertex.
<p>
   A line in an OBJ file that begins with {@code 'f'} is a line that
   describes a "face". The {@code 'f'} will be followed on the line by
   a sequence of positive integers. The integers are the indices of the
   vertices that make up the face. The "index" of a vertex is the order
   in which the vertex was listed in the OBJ file. So a line like this
<pre>{@code
      f  2  4  1
}</pre>
   would represent a triangle made up of the 2nd vertex read from the file,
   the 4th vertex read from the file, and the 1st vertex read from the file.
   And a line like this
<pre>{@code
      f  2  4  3  5
}</pre>
   would represent a quadrilateral made up of the 2nd vertex read from the file,
   the 4th vertex read from the file, the 3rd vertex read from the file, and
   the 5th vertex read from the file.
<p>
   See <a href="https://en.wikipedia.org/wiki/Wavefront_.obj_file" target="_top">
                https://en.wikipedia.org/wiki/Wavefront_.obj_file</a>
*/
public class PointCloud_ObjSimpleModelSolid extends Model
{
   /**
      Create a solid point cloud model from the contents of an OBJ file.

      @param objFile  {@link File} object for the OBJ data file
      @param n        number of points in each line segment
   */
   public PointCloud_ObjSimpleModelSolid(final File objFile, final int n)
   {
      super("Point Cloud OBJ Model Solid");

      // Open the OBJ file.
      String objName = null;
      FileInputStream fis = null;
      try
      {
         objName = objFile.getCanonicalPath();
         fis = new FileInputStream( objFile );
      }
      catch (FileNotFoundException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }

      this.name += (", " + objName);

      // Get the geometry from the OBJ file.
      try
      {
         // Pattern for parsing lines that start with "f"
         final Pattern p = Pattern.compile("^(\\d*)[/]?(\\d*)[/]?(\\d*)");

         final Scanner scanner = new Scanner(fis);
         while ( scanner.hasNext() )
         {
            final String token = scanner.next();
            if ( token.startsWith("#")
              || token.startsWith("vt")
              || token.startsWith("vn")
              || token.startsWith("s")
              || token.startsWith("g")
              || token.startsWith("o")
              || token.startsWith("usemtl")
              || token.startsWith("mtllib") )
            {
               scanner.nextLine(); // skip over these lines
            }
            else if ( token.startsWith("v") )
            {
               final double x = scanner.nextDouble();
               final double y = scanner.nextDouble();
               final double z = scanner.nextDouble();
               addVertex( new Vertex(x, y, z) );
            }// parse vertex
            else if ( token.startsWith("f") )
            {
               // tokenize the rest of the line
               final String restOfLine = scanner.nextLine();
               final Scanner scanner2 = new Scanner( restOfLine );
               // parse three vertices and fill in a triangle shape
               final int[] vIndex = new int[3];
               for (int i = 0; i < 3; ++i)
               {
                  // parse a "v/vt/vn" group
                  final String faceGroup = scanner2.next();
                  final Matcher m = p.matcher( faceGroup );
                  if ( m.find() )
                  {
                     vIndex[i] = Integer.parseInt(m.group(1)) - 1;
                     final String vt = m.group(2);  // don't need
                     final String vn = m.group(3);  // don't need
                  }
                  else
                     System.err.println("Error: bad face: " + faceGroup);
               }
               fillInTriangleB(vIndex[0], vIndex[1], vIndex[2], n);


               // parse another vertex (if there is one) and fill in another triangle
               while (scanner2.hasNext())
               {
                  vIndex[1] = vIndex[2];
                  final String faceGroup = scanner2.next();
                  final Matcher m = p.matcher( faceGroup );
                  if ( m.find() )
                  {
                     vIndex[2] = Integer.parseInt(m.group(1)) - 1;
                     final String vt = m.group(2);  // don't need
                     final String vn = m.group(3);  // don't need
                  }
                  else
                     System.err.println("Error: bad face: " + faceGroup);

                  fillInTriangleB(vIndex[0],
                                  vIndex[1],
                                  vIndex[2], n);
               }
            }// parse face
         }// parse one line
         fis.close();
      }
      catch (Exception e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not read OBJ file: %s\n", objName);
         System.exit(-1);
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }


   /**
      Recursively use Barycentric subdivision to put into this
      {@link Model} points that fill in the triangular space
      between the {@link Vertex} indexed by {@code vIndex0},
      {@code vIndex1} and {@code vIndex2}.
      <p>
      The value of {@code n} should be less than 6 or 7.

      @param vIndex0  index of a {link Vertex} of a triangle
      @param vIndex1  index of a {link Vertex} of a triangle
      @param vIndex2  index of a {link Vertex} of a triangle
      @param n   number of Barycentric subdivision of this triangle
   */
   private void fillInTriangleB(final int vIndex0,
                                final int vIndex1,
                                final int vIndex2,
                                final int n)
   {
      if (n > 0)
      {
         final Vertex v0 = vertexList.get(vIndex0);
         final Vertex v1 = vertexList.get(vIndex1);
         final Vertex v2 = vertexList.get(vIndex2);
         fillInTriangleB(v0, v1, v2, n);
      }
   }


   /**
      Recursively use Barycentric subdivision to put into this
      {@link Model} points that fill in the triangular space
      between {@link Vertex} {@code v0}, {@code v1} and {@code v2}.
      <p>
      For OBJ files, the value of {@code n} should be 2 or 3.

      @param v0  {link Vertex} of a triangle
      @param v1  {link Vertex} of a triangle
      @param v2  {link Vertex} of a triangle
      @param n   number of Barycentric subdivisions of this triangle
   */
   private void fillInTriangleB(final Vertex v0,
                                final Vertex v1,
                                final Vertex v2,
                                final int n)
   {
      if (n > 0)
      {
         // Barycentric subdivision
         // https://en.wikipedia.org/wiki/Barycentric_subdivision
         final Vertex vCenter = new Vertex(
         //         (1/3)*v0 + (1/3)*v1 + (1/3)*v2
                    (v0.x + v1.x + v2.x)/3.0,
                    (v0.y + v1.y + v2.y)/3.0,
                    (v0.z + v1.z + v2.z)/3.0);
         final Vertex v01 = new Vertex(
         //         (1/2)*v0 + (1/2)*v1
                    (v0.x + v1.x)/2.0,
                    (v0.y + v1.y)/2.0,
                    (v0.z + v1.z)/2.0);
         final Vertex v12 = new Vertex(
         //         (1/2)*v1 + (1/2)*v2
                    (v1.x + v2.x)/2.0,
                    (v1.y + v2.y)/2.0,
                    (v1.z + v2.z)/2.0);
         final Vertex v20 = new Vertex(
         //         (1/2)*v2 + (1/2)*v0
                    (v2.x + v0.x)/2.0,
                    (v2.y + v0.y)/2.0,
                    (v2.z + v0.z)/2.0);
         addVertex( vCenter );
         addEdge(v0,  vCenter, 10);
         addEdge(v1,  vCenter, 10);
         addEdge(v2,  vCenter, 10);
         addEdge(v01, vCenter, 10);
         addEdge(v12, vCenter, 10);
         addEdge(v20, vCenter, 10);
         fillInTriangleB(v0, v01, vCenter, n-1);
         fillInTriangleB(v0, v20, vCenter, n-1);
         fillInTriangleB(v1, v01, vCenter, n-1);
         fillInTriangleB(v1, v12, vCenter, n-1);
         fillInTriangleB(v2, v12, vCenter, n-1);
         fillInTriangleB(v2, v20, vCenter, n-1);
      }
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between the {@link Vertex} indexed by {@code vIndex0}
      and the {@link Vertex} indexed by {@code vIndex1}.

      @param vIndex0  index of starting {link Vertex} of a line segment
      @param vIndex1  index of ending {link Vertex} of a line segment
      @param n        number of points for this edge
   */
   private void addEdge(final int vIndex0, final int vIndex1, final int n)
   {
      final Vertex v0 = vertexList.get(vIndex0);
      final Vertex v1 = vertexList.get(vIndex1);
      addEdge(v0, v1, n);
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between {@link Vertex} {@code v0} and {@link Vertex}
      {@code v1}.

      @param v0  starting {link Vertex} of a line segment
      @param v1  ending {link Vertex} of a line segment
      @param n   number of points for this edge
   */
   private void addEdge(final Vertex v0, final Vertex v1, final int n)
   {
      final double deltaT = 1.0 / (n - 1);

      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         t += deltaT;
      }
   }
}//PointCloud_ObjSimpleModelSolid
