/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a regular octahedron
   with its center at the origin, having side length
   {@code  sqrt(2) = 1.4142},with its center plane given
   by the four vertices {@code  (1, 0, 1)}. and with
   the top and bottom vertices being {@code  (0, 1, 0)}.
<p>
   See <a href="http://en.wikipedia.org/wiki/Octahedron" target="_top">
                http://en.wikipedia.org/wiki/Octahedron</a>

   @see PointCloud_Tetrahedron
   @see PointCloud_Cube
   @see PointCloud_Icosahedron
   @see PointCloud_Dodecahedron
*/
public class PointCloud_Octahedron extends Model
{
   /**
      Create a regular octahedron with its center at the
      origin, having side length {@code  sqrt(2) = 1.4142},
      with its center plane given by the four vertices
      {@code  (1, 0, 1)}. and with the top and bottom
      vertices being {@code  (0, 1, 0)}.
   */
   public PointCloud_Octahedron()
   {
      this(100);
   }


   /**
      Create a regular octahedron with {@code n}
      points along each edge.

      @param n  number of points along each edge
   */
   public PointCloud_Octahedron(final int n)
   {
      super("Point Cloud Octahedron");

      // Create the octahedron's geometry.
      // It has 6 vertices and 12 edges.
      Vertex v0, v1, v2, v3, v4, v5;

      v0 = new Vertex( 1,  0,  0); // 4 vertices around the center plane
      v1 = new Vertex( 0,  0, -1);
      v2 = new Vertex(-1,  0,  0);
      v3 = new Vertex( 0,  0,  1);
      v4 = new Vertex( 0,  1,  0); // vertex at the top
      v5 = new Vertex( 0, -1,  0); // vertex at the bottom
/*
      // These vertices create an Octahedron with side length 1.
      final double sqrt3 = Math.sqrt(3.0);
      final double sqrt2 = Math.sqrt(2.0);
      v0 = new Vertex( 0.5, 0,  0.5); // 4 vertices around the center plane
      v1 = new Vertex(-0.5, 0,  0.5);
      v2 = new Vertex(-0.5, 0, -0.5);
      v3 = new Vertex( 0.5, 0, -0.5);
      v4 = new Vertex( 0,  1/sqrt2, 0); // vertex at the top
      v5 = new Vertex( 0, -1/sqrt2, 0); // vertex at the bottom
*/
      addVertex(v0, v1, v2, v3, v4, v5);

      // Create point clouds for 12 edges.
      // four line segments around the center plane
      addEdge(v0, v1, n);
      addEdge(v1, v2, n);
      addEdge(v2, v3, n);
      addEdge(v3, v0, n);
      // edges going to the top vertex
      addEdge(v0, v4, n);
      addEdge(v1, v4, n);
      addEdge(v2, v4, n);
      addEdge(v3, v4, n);
      // edges going to the bottom vertex
      addEdge(v0, v5, n);
      addEdge(v1, v5, n);
      addEdge(v2, v5, n);
      addEdge(v3, v5, n);

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between {@link Vertex} {@code v0} and {@link Vertex}
      {@code v1}.

      @param v0  starting {link Vertex} of a line segment
      @param v1  ending {link Vertex} of a line segment
      @param n   number of points for this edge
   */
   private void addEdge(final Vertex v0, final Vertex v1, final int n)
   {
      final double deltaT = 1.0 / (n - 1);

      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         t += deltaT;
      }
   }
}//PointCloud_Octahedron
