/*

*/

package models_pc;
import  renderer.scene.*;

import java.util.function.DoubleFunction;
import java.util.function.ToDoubleFunction;    // could use this instead
import java.util.function.DoubleUnaryOperator; // could use this instead
//https://docs.oracle.com/javase/8/docs/api/java/util/function/package-summary.html

/**
   Create a point cloud model of a parametric curve in space.
<p>
   See <a href="https://en.wikipedia.org/wiki/Parametric_equation" target="_top">
                https://en.wikipedia.org/wiki/Parametric_equation</a>

   @see PointCloud_ParametricSurface
*/
public class PointCloud_ParametricCurve extends Model
{
   /**
      Create a trefoil knot as a parametric curve in space.
   <p>
      See <a href="https://en.wikipedia.org/wiki/Trefoil_knot#Descriptions" target="_top">
                   https://en.wikipedia.org/wiki/Trefoil_knot#Descriptions</a>
   */
   public PointCloud_ParametricCurve()
   {
      this(t ->  0.5*Math.sin(t) + Math.sin(2*t),
           t ->  0.5*Math.cos(t) - Math.cos(2*t),
           t -> -0.5*Math.sin(3*t),
           0, 2*Math.PI, 1000);
   }


   /**
      Create a parametric curve in the xy-plane,
      <pre>{@code
         x = x(t)
         y = y(t)
      }</pre>
      with the parameter {@code  t} having the given parameter
      range and the given number of points along the curve.

      @param x   component function in the x-direction
      @param y   component function in the y-direction
      @param t1  beginning value of parameter range
      @param t2  ending value of parameter range
      @param n   number of points in the curve
   */
   public PointCloud_ParametricCurve(final DoubleFunction<Double> x,
                                     final DoubleFunction<Double> y,
                                     final double t1, final double t2,
                                     final int n)
   {
      this(x, y, t->0.0, t1, t2, n);
   }


   /**
      Create a parametric curve in space,
      <pre>{@code
         x = x(t)
         y = y(t)
         z = z(t)
      }</pre>
      with the parameter {@code t} having the given parameter
      range and the given number of points along the curve.

      @param x   component function in the x-direction
      @param y   component function in the y-direction
      @param z   component function in the z-direction
      @param t1  beginning value of parameter range
      @param t2  ending value of parameter range
      @param n   number of points in the curve
   */
   public PointCloud_ParametricCurve(final DoubleFunction<Double> x,
                                     final DoubleFunction<Double> y,
                                     final DoubleFunction<Double> z,
                                     final double t1, final double t2,
                                     final int n)
   {
      super("Point Cloud Parametric Curve");

      // Create the curve's geometry.
      final double deltaT = (t2 - t1) / n;

      // Vertices.
      for (int i = 0; i <= n; ++i)
      {
         addVertex(new Vertex(x.apply(t1 + i * deltaT),
                              y.apply(t1 + i * deltaT),
                              z.apply(t1 + i * deltaT)));
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); i++)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_ParametricCurve
