/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a square in the xy-plane centered at the origin.
<p>
   Here is a picture showing the square's four corners.
<pre>{@code
                   y
                   |
                   |
 (-r, r)           |           (r, r)
        +----------------------+
        |          |           |
        |          |           |
        |          |           |
        |          |           |
  ------|----------+-----------|-------> x
        |          |           |
        |          |           |
        |          |           |
        +----------------------+
 (-r, -r)          |           (r, -r)
                   |
                   |
}</pre>
*/
public class PointCloud_Square extends Model
{
   /**
      Create a square in the xy-plane with corners {@code (1, 1, 0)}
      and with 100 points along each edge of the square.
   */
   public PointCloud_Square( )
   {
      this(1.0, 100);
   }


   /**
      Create a square in the xy-plane with corners {@code (r, r, 0)}
      and with 100 points along each edge of the square.

      @param r  determines the corners of the square
   */
   public PointCloud_Square(final double r)
   {
      this(r, 100);
   }


   /**
      Create a square in the xy-plane with corners {@code (r, r, 0)}
      and with {@code n} points along each edge of the square.

      @param r  determines the corners of the square
      @param n  number of points along each edge of the square
   */
   public PointCloud_Square(double r, final int n)
   {
      super("Point Cloud Square (n="+n+")");

      r = Math.abs(r);

      // Create the geometry for the Model.
      final double delta = (2*r)/n;

      // Vertices.
      for (int i = 0; i < n; ++i)
      {
         addVertex(new Vertex( r,            r - i*delta, 0),
                   new Vertex( r - i*delta, -r,           0),
                   new Vertex(-r,           -r + i*delta, 0),
                   new Vertex(-r + i*delta,  r,           0));
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Square
