/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a regular tetrahedron
   with its center at the origin, having edge length
   {@code 2*sqrt(2)}, and with its vertices at corners
   of the cube with vertices {@code (1, 1, 1)}.
<p>
   See <a href="http://en.wikipedia.org/wiki/Tetrahedron" target="_top">
                http://en.wikipedia.org/wiki/Tetrahedron</a>

   @see PointCloud_Cube
   @see PointCloud_Octahedron
   @see PointCloud_Icosahedron
   @see PointCloud_Dodecahedron
*/
public class PointCloud_Tetrahedron extends Model
{
   /**
      Create a regular tetrahedron with its center at
      the origin, having edge length {@code 2*sqrt(2)},
      and with its vertices at corners of the cube with
      vertices {@code (1, 1, 1)}.
   */
   public PointCloud_Tetrahedron()
   {
      this(false, 100);
   }


   /**
      Create a regular tetrahedron with {@code n}
      points along each edge.

      @param n  number of points along each edge
   */
   public PointCloud_Tetrahedron(final int n)
   {
      this(false, n);
   }


   /**
      Create a regular tetrahedron or its dual tetrahedron
      (the dual of a tetrahedron is another tetrahedron).

      @param dual  choose between the two dual tetrahedrons
      @param n     number of points along an edge
   */
   public PointCloud_Tetrahedron(final boolean dual, final int n)
   {
      super("Point Cloud Tetrahedron");

      final double deltaT = 1.0 / (n - 1);

      // Create the tetrahedron's geometry.
      // It has 4 vertices and 6 edges.
      Vertex v0, v1, v2, v3;
      if ( ! dual)
      {
         v0 = new Vertex( 1,  1,  1);
         v1 = new Vertex(-1,  1, -1);
         v2 = new Vertex( 1, -1, -1);
         v3 = new Vertex(-1, -1,  1);
      }
      else // create the dual tetrahedron by
      {    // inverting the coordinates given above
         v0 = new Vertex(-1, -1, -1);
         v1 = new Vertex( 1, -1,  1);
         v2 = new Vertex(-1,  1,  1);
         v3 = new Vertex( 1,  1, -1);
      }
      addVertex(v0, v1, v2, v3);

      // Create point clouds for 6 edges.
      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         addVertex( new Vertex(
         //         (1-t)*v2  +  t*v3
                    (1-t)*v2.x + t*v3.x,
                    (1-t)*v2.y + t*v3.y,
                    (1-t)*v2.z + t*v3.z ));
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v2
                    (1-t)*v0.x + t*v2.x,
                    (1-t)*v0.y + t*v2.y,
                    (1-t)*v0.z + t*v2.z ));
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v3
                    (1-t)*v0.x + t*v3.x,
                    (1-t)*v0.y + t*v3.y,
                    (1-t)*v0.z + t*v3.z ));
         addVertex( new Vertex(
         //         (1-t)*v1  +  t*v2
                    (1-t)*v1.x + t*v2.x,
                    (1-t)*v1.y + t*v2.y,
                    (1-t)*v1.z + t*v2.z ));
         addVertex( new Vertex(
         //         (1-t)*v1  +  t*v3
                    (1-t)*v1.x + t*v3.x,
                    (1-t)*v1.y + t*v3.y,
                    (1-t)*v1.z + t*v3.z ));
         t += deltaT;
      }

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }
}//PointCloud_Tetrahedron
