/*

*/

package models_pc;
import  renderer.scene.*;

/**
   Create a point cloud model of a regular tetrahedron
   with its center at the origin, having edge length
   {@code 2*sqrt(2)}, and with its vertices at corners
   of the cube with vertices {@code (1, 1, 1)}.
*/
public class PointCloud_TetrahedronSolid extends Model
{
   /**
      Create a regular tetrahedron with its center at
      the origin, having edge length {@code 2*sqrt(2)},
      and with its vertices at corners of the cube with
      vertices {@code (1, 1, 1)}.
   */
   public PointCloud_TetrahedronSolid()
   {
      this(false, 100);
   }


   /**
      Create a regular tetrahedron with {@code n}
      points along each edge.

      @param n  number of points along each edge
   */
   public PointCloud_TetrahedronSolid(final int n)
   {
      this(false, n);
   }


   /**
      Create a regular tetrahedron or its dual tetrahedron
      (the dual of a tetrahedron is another tetrahedron).

      @param dual  choose between the two dual tetrahedrons
      @param n     number of points along an edge
   */
   public PointCloud_TetrahedronSolid(final boolean dual, final int n)
   {
      super("Point Cloud Tetrahedron Solid");

      final double deltaT = 1.0 / (n - 1);

      // Create the tetrahedron's geometry.
      // It has 4 vertices, 6 edges, and 4 faces.
      Vertex v0, v1, v2, v3;
      if ( ! dual)
      {
         v0 = new Vertex( 1,  1,  1);
         v1 = new Vertex(-1,  1, -1);
         v2 = new Vertex( 1, -1, -1);
         v3 = new Vertex(-1, -1,  1);
      }
      else // create the dual tetrahedron by
      {    // inverting the coordinates given above
         v0 = new Vertex(-1, -1, -1);
         v1 = new Vertex( 1, -1,  1);
         v2 = new Vertex(-1,  1,  1);
         v3 = new Vertex( 1,  1, -1);
      }
      addVertex(v0, v1, v2, v3);

      // Create point clouds for 4 faces.
      fillInTriangleB(v0, v1, v2, n); // use fillInTriangleA(),
      fillInTriangleB(v0, v1, v3, n); //  or fillInTriangleC()
      fillInTriangleB(v1, v2, v3, n);
      fillInTriangleB(v2, v0, v3, n);
      addEdge(v0, v1, 100);
      addEdge(v1, v2, 100);
      addEdge(v2, v0, 100);
      addEdge(v0, v3, 100);
      addEdge(v1, v3, 100);
      addEdge(v2, v3, 100);

      // Line segments.
      for (int i = 0; i < vertexList.size(); ++i)
      {
         // create line segments that are a single point
         addLineSegment( new LineSegment(i, i) );
      }
   }


   /**
      Recursively put into this {@link Model} points that
      fill in the triangular space between {@link Vertex}
      {@code v0}, {@code v1} and {@code v2}.

      @param v0  {link Vertex} of a triangle
      @param v1  {link Vertex} of a triangle
      @param v2  {link Vertex} of a triangle
      @param n   number of levels of recursion
   */
   private void fillInTriangleA(final Vertex v0,
                                final Vertex v1,
                                final Vertex v2,
                                final int n)
   {
      if (n > 0)
      {
         final Vertex vCenter = new Vertex(
         //         (1/3)*v0 + (1/3)*v1 + (1/3)*v2
                    (v0.x + v1.x + v2.x)/3.0,
                    (v0.y + v1.y + v2.y)/3.0,
                    (v0.z + v1.z + v2.z)/3.0);
         addVertex( vCenter );
         addEdge(v0, vCenter, 10);
         addEdge(v1, vCenter, 10);
         addEdge(v2, vCenter, 10);
         fillInTriangleA(v0, v1, vCenter, n-1);
         fillInTriangleA(v1, v2, vCenter, n-1);
         fillInTriangleA(v2, v0, vCenter, n-1);
      }
   }


   /**
      Recursively use Barycentric subdivision to put into this
      {@link Model} points that fill in the triangular space
      between {@link Vertex} {@code v0}, {@code v1} and {@code v2}.
      <p>
      The value of {@code n} should be less than 6 or 7.

      @param v0  {link Vertex} of a triangle
      @param v1  {link Vertex} of a triangle
      @param v2  {link Vertex} of a triangle
      @param n   number of Barycentric subdivisions of this triangle
   */
   private void fillInTriangleB(final Vertex v0,
                                final Vertex v1,
                                final Vertex v2,
                                final int n)
   {
      if (n > 0)
      {
         // Barycentric subdivision
         // https://en.wikipedia.org/wiki/Barycentric_subdivision
         final Vertex vCenter = new Vertex(
         //         (1/3)*v0 + (1/3)*v1 + (1/3)*v2
                    (v0.x + v1.x + v2.x)/3.0,
                    (v0.y + v1.y + v2.y)/3.0,
                    (v0.z + v1.z + v2.z)/3.0);
         final Vertex v01 = new Vertex(
         //         (1/2)*v0 + (1/2)*v1
                    (v0.x + v1.x)/2.0,
                    (v0.y + v1.y)/2.0,
                    (v0.z + v1.z)/2.0);
         final Vertex v12 = new Vertex(
         //         (1/2)*v1 + (1/2)*v2
                    (v1.x + v2.x)/2.0,
                    (v1.y + v2.y)/2.0,
                    (v1.z + v2.z)/2.0);
         final Vertex v20 = new Vertex(
         //         (1/2)*v2 + (1/2)*v0
                    (v2.x + v0.x)/2.0,
                    (v2.y + v0.y)/2.0,
                    (v2.z + v0.z)/2.0);
         addVertex( vCenter );
         addEdge(v0,  vCenter, 10);
         addEdge(v1,  vCenter, 10);
         addEdge(v2,  vCenter, 10);
         addEdge(v01, vCenter, 10);
         addEdge(v12, vCenter, 10);
         addEdge(v20, vCenter, 10);
         fillInTriangleB(v0, v01, vCenter, n-1);
         fillInTriangleB(v0, v20, vCenter, n-1);
         fillInTriangleB(v1, v01, vCenter, n-1);
         fillInTriangleB(v1, v12, vCenter, n-1);
         fillInTriangleB(v2, v12, vCenter, n-1);
         fillInTriangleB(v2, v20, vCenter, n-1);
      }
   }


   /**
      Put into this {@link Model} points that fill in the
      triangular space between {@link Vertex} {@code v0},
      {@code v1} and {@code v2} with concentric triangles.

      @param v0  {link Vertex} of a triangle
      @param v1  {link Vertex} of a triangle
      @param v2  {link Vertex} of a triangle
      @param n   number of concentric triangles
   */
   private void fillInTriangleC(final Vertex v0,
                                final Vertex v1,
                                final Vertex v2,
                                final int n)
   {
      if (n > 0)
      {
         addEdge(v0, v1, 100);
         addEdge(v1, v2, 100);
         addEdge(v2, v0, 100);
         final Vertex w0 = new Vertex(
         //         (9/10)*v0 + (1/20)*v1 + (1/20)*v2
                    (0.9*v0.x + 0.05*v1.x + 0.05*v2.x),
                    (0.9*v0.y + 0.05*v1.y + 0.05*v2.y),
                    (0.9*v0.z + 0.05*v1.z + 0.05*v2.z));
         final Vertex w1 = new Vertex(
         //         (1/20)*v0 + (9/10)*v1 + (1/20)*v2
                    (0.05*v0.x + 0.9*v1.x + 0.05*v2.x),
                    (0.05*v0.y + 0.9*v1.y + 0.05*v2.y),
                    (0.05*v0.z + 0.9*v1.z + 0.05*v2.z));
         final Vertex w2 = new Vertex(
         //         (1/20)*v0 + (1/20)*v1 + (9/10)*v2
                    (0.05*v0.x + 0.05*v1.x + 0.9*v2.x),
                    (0.05*v0.y + 0.05*v1.y + 0.9*v2.y),
                    (0.05*v0.z + 0.05*v1.z + 0.9*v2.z));
         addVertex(w0, w1, w2);
         fillInTriangleC(w0, w1, w2, n-1);
      }
   }


   /**
      Put into this {@link Model} the {@code n-2} points needed for
      the edge between {@link Vertex} {@code v0} and {@link Vertex}
      {@code v1}.

      @param v0  starting {link Vertex} of a line segment
      @param v1  ending {link Vertex} of a line segment
      @param n   number of points for this edge
   */
   private void addEdge(final Vertex v0, final Vertex v1, final int n)
   {
      final double deltaT = 1.0 / (n - 1);

      double t = deltaT;
      for (int i = 1; i < n - 1; ++i)
      {
         // use linear interpolation (lerp)
         addVertex( new Vertex(
         //         (1-t)*v0  +  t*v1
                    (1-t)*v0.x + t*v1.x,
                    (1-t)*v0.y + t*v1.y,
                    (1-t)*v0.z + t*v1.z ));
         t += deltaT;
      }
   }
}//PointCloud_TetrahedronSolid
